<?php

declare(strict_types=1);

namespace Drupal\message_recipient_group\Plugin\MessageRecipient\RecipientsCollector;

use Drupal\Core\Entity\Query\QueryInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\group\Entity\GroupInterface;
use Drupal\group\Entity\GroupMembership;
use Drupal\group\Entity\GroupMembershipInterface;
use Drupal\group\Entity\GroupRelationship;
use Drupal\group\Entity\GroupRelationshipInterface;
use Drupal\message\MessageInterface;
use Drupal\message\MessageTemplateInterface;
use Drupal\message_recipient\Attribute\RecipientsCollector;
use Drupal\message_recipient_entity\ContentEntityUserRecipientsCollectorBase;

/**
 * Users by group roles recipients' collector.
 */
#[RecipientsCollector(
  id: 'users_by_group_roles',
  type: 'user',
  category: 'group',
  label: new TranslatableMarkup('Users by group roles'),
  description: new TranslatableMarkup('Collects users by group roles from the message\'s associated "group/group relationship" entity.'),
)]
class UsersByGroupRoles extends ContentEntityUserRecipientsCollectorBase {

  /**
   * The user ids.
   *
   * @var array
   */
  protected array $userIds = [];

  /**
   * {@inheritdoc}
   */
  public function collect(MessageInterface $message): array {
    $groups = $this->getGroupByMessage($message);
    foreach ($groups as $group) {
      $ids = array_map(fn(GroupMembershipInterface $membership) => $membership->getEntity()->id(),
        GroupMembership::loadByGroup($group, $this->configuration['group_roles'][$group->getGroupType()->id()]));

      $this->userIds = array_merge($this->userIds, $ids);
    }

    return !empty($this->userIds)
      ? array_values($this->query($message)->execute())
      : [];
  }

  /**
   * {@inheritdoc}
   */
  public function query(MessageInterface $message): QueryInterface {
    $query = parent::query($message);
    $query->condition('uid', $this->userIds, 'IN');
    return $query;
  }

  /**
   * {@inheritdoc}
   */
  public function isApplicable(MessageInterface $message): bool {
    if ($this->hasAssociatedEntityField($message) && !empty($this->configuration['group_roles'])) {
      $groups = $this->getGroupByMessage($message);
      return !empty($groups);
    }
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    return [
      'group_roles' => [],
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state, MessageTemplateInterface $message_template): array {
    $form['description'] = [
      '#type' => 'item',
      '#title' => $this->t('Set group roles per group type for this recipients collector:'),
    ];

    $group_types = $this->entityTypeManager->getStorage('group_type')->loadMultiple();
    $form['group_roles'] = [
      '#type' => 'container',
      '#tree' => TRUE,
    ];
    foreach ($group_types as $group_type) {
      /** @var \Drupal\group\Entity\GroupTypeInterface $group_type */
      $group_roles = $group_type->getRoles();
      $options = [];
      foreach ($group_roles as $group_role) {
        if ($group_role->isAnonymous()) {
          continue;
        }
        $options[$group_role->id()] = $group_role->label();
      }
      $form['group_roles'][$group_type->id()] = [
        '#type' => 'checkboxes',
        '#title' => $this->t('%group_type roles:', ['%group_type' => $group_type->label()]),
        '#default_value' => $this->configuration['group_roles'][$group_type->id()] ?? [],
        '#options' => $options,
      ];
    }
    return $form + parent::buildConfigurationForm($form, $form_state, $message_template);
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state, MessageTemplateInterface $message_template): void {
    parent::validateConfigurationForm($form, $form_state, $message_template);
    if (empty($this->prepareRoles($form_state->getValue('group_roles')))) {
      $form_state->setErrorByName('', $this->t('You should select at least one role.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state, MessageTemplateInterface $message_template): void {
    parent::submitConfigurationForm($form, $form_state, $message_template);
    $this->configuration['group_roles'] = $this->prepareRoles($form_state->getValue('group_roles'));
  }

  /**
   * {@inheritdoc}
   */
  public function getSummary(): array {
    $summary = [
      '#markup' => $this->t('Collects users by the following group roles from the message\'s associated "group/group relationship" entity:'),
    ];
    $roles_summary = [];
    foreach ($this->configuration['group_roles'] as $group_type => $group_roles) {
      /** @var \Drupal\group\Entity\GroupTypeInterface $group_type */
      $group_type = $this->entityTypeManager->getStorage('group_type')->load($group_type);
      $role_names = [];
      foreach ($group_roles as $group_role) {
        /** @var \Drupal\group\Entity\GroupRoleInterface $group_role */
        $group_role = $this->entityTypeManager->getStorage('group_role')->load($group_role);
        $role_names[] = $group_role->label();
      }
      $roles_summary[] = '<br/><strong>' . $group_type->label() . ':</strong> ' . implode(', ', $role_names);
    }
    $summary[] = [
      '#markup' => implode('', $roles_summary),
    ];

    return [
      'recipient_collector' => $summary,
    ] + parent::getSummary();
  }

  /**
   * Prepare roles for saving.
   *
   * @param array $unfiltered_roles
   *   The group roles grouped by group types.
   *
   * @return array
   *   List of group roles machine names grouped by group types.
   */
  protected function prepareRoles(array $unfiltered_roles): array {
    $roles = [];
    foreach ($unfiltered_roles as $group_type => $group_roles) {
      $filtered_roles = array_values(array_filter($group_roles));
      if (!empty($filtered_roles)) {
        $roles[$group_type] = $filtered_roles;
      }
    }
    return $roles;
  }

  /**
   * Get groups by message's associated entity.
   *
   * @param \Drupal\message\MessageInterface $message
   *   The message entity.
   *
   * @return \Drupal\group\Entity\GroupInterface[]
   *   List group entities if found.
   */
  protected function getGroupByMessage(MessageInterface $message): array {
    $associated_entity = $this->getAssociatedEntity($message);
    if ($associated_entity instanceof GroupInterface) {
      return [$associated_entity];
    }
    elseif ($associated_entity instanceof GroupRelationshipInterface) {
      return [$this->getAssociatedEntity($message)->getGroup()];
    }
    elseif ($group_relationships = GroupRelationship::loadByEntity($associated_entity)) {
      $groups = [];
      foreach ($group_relationships as $group_relationship) {
        $groups[] = $group_relationship->getGroup();
      }

      return $groups;
    }
    return [];
  }

}
